import torch.nn as nn
import torch

class Convnet(nn.Module):
    def __init__(self):
        super(Convnet, self).__init__()
        #  b, 2, 3, 3
        self.feature = nn.Sequential(
            nn.Conv2d(in_channels=2, out_channels=16, kernel_size=3, padding=1),
            nn.BatchNorm2d(num_features=16),
            nn.ReLU(inplace=True),   # 16, 3, 3
            nn.Conv2d(in_channels=16, out_channels=32, kernel_size=3, padding=1),
            nn.BatchNorm2d(num_features=32),
            nn.ReLU(inplace=True),  # 32, 3, 3
            nn.Conv2d(in_channels=32, out_channels=64, kernel_size=3, padding=1),
            nn.BatchNorm2d(num_features=64),
            nn.ReLU(inplace=True)  # 64, 3, 3
        )
        self.linear = nn.Sequential(
            nn.Linear(in_features=256, out_features=128),
            nn.BatchNorm1d(num_features=128),
            nn.ReLU(True),
            nn.Linear(in_features=128, out_features=64),
            nn.BatchNorm1d(num_features=64),
            nn.ReLU(True),
            nn.Dropout(),
            nn.Linear(64, 1)
        )
    def forward(self, x):
        x = self.feature(x)
        x = self.spatial_pyramid_pool(x, x.size(0), (x.size(2), x.size(3)), [2])
        # print("x.size()=", x.size())
        x = self.linear(x)  # b
        return x.squeeze()

    def spatial_pyramid_pool(self, previous_conv, num_sample, previous_conv_size, out_pool_size):
        '''
        previous_conv: a tensor vector of previous convolution layer
        num_sample: an int number of image in the batch
        previous_conv_size: an int vector [height, width] of the matrix features size of previous convolution layer
        out_pool_size: a int vector of expected output size of max pooling layer

        returns: a tensor vector with shape [1 x n] is the concentration of multi-level pooling
        '''
        # print(previous_conv.size())
        import math
        for i in range(len(out_pool_size)):
            # print(previous_conv_size)
            h_wid = int(math.ceil(previous_conv_size[0] / out_pool_size[i]))
            w_wid = int(math.ceil(previous_conv_size[1] / out_pool_size[i]))
            h_pad = int((h_wid * out_pool_size[i] - previous_conv_size[0] + 1) / 2)
            w_pad = int((w_wid * out_pool_size[i] - previous_conv_size[1] + 1) / 2)
            maxpool = nn.MaxPool2d((h_wid, w_wid), stride=(h_wid, w_wid), padding=(h_pad, w_pad))
            x = maxpool(previous_conv)
            if (i == 0):
                spp = x.view(num_sample, -1)
                # print("spp size:",spp.size())
            else:
                # print("size:",spp.size())
                spp = torch.cat((spp, x.view(num_sample, -1)), 1)
        return spp

# class Convnet(nn.Module):
#     def __init__(self):
#         super(Convnet, self).__init__()
#         #1, 3, 3
#         self.feature = nn.Sequential(
#             nn.Conv2d(in_channels=1, out_channels=4, kernel_size=3, padding=1),
#             nn.BatchNorm2d(num_features=4),
#             nn.ReLU(inplace=True),   # 4, 3, 3
#             nn.Conv2d(in_channels=4, out_channels=8, kernel_size=3, padding=1),
#             nn.BatchNorm2d(num_features=8),
#             nn.ReLU(inplace=True)  # 8, 3, 3
#         )
#         self.linear = nn.Sequential(
#             nn.Linear(in_features=8*3*3, out_features=128),
#             nn.BatchNorm1d(num_features=128),
#             nn.ReLU(True),
#             nn.Linear(in_features=128, out_features=64),
#             nn.BatchNorm1d(num_features=64),
#             nn.ReLU(True),
#             nn.Dropout(),
#             nn.Linear(64, 9)
#         )
#     def forward(self, x):
#         x = self.feature(x)
#         x = x.view(x.size(0), -1)
#         x = self.linear(x)  # b, 9
#         return x
